/*******************************************************************************
 * Copyright (c) 2010- Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.datasource;

import org.simantics.databoard.accessor.Accessor;
import org.simantics.databoard.accessor.ArrayAccessor;
import org.simantics.databoard.accessor.MapAccessor;
import org.simantics.databoard.accessor.RecordAccessor;
import org.simantics.databoard.accessor.error.AccessorConstructionException;
import org.simantics.databoard.accessor.error.AccessorException;
import org.simantics.databoard.binding.Binding;
import org.simantics.databoard.binding.mutable.Variant;
import org.simantics.databoard.binding.mutable.VariantContainerBinding;

public class DatasourceUtil {

	static final Binding ID_BINDING = VariantContainerBinding.INSTANCE;
	
	/** The identifier or the root node in all data sources */
	public static final Variant ROOT_ID = (Variant) ID_BINDING.createDefaultUnchecked();
	
	/**
	 * Get children of a node in a accessor of a datasource.
	 * 
	 * @param accessor
	 * @param nodeId
	 * @return child keys
	 * @throws AccessorException
	 */
	public static Variant[] getChildren(Accessor accessor, Variant nodeId)
	throws AccessorException {
		try {
			RecordAccessor ra = (RecordAccessor) accessor;
			MapAccessor ma = ra.getFieldAccessor("nodes");
			RecordAccessor na = ma.getValueAccessor(ID_BINDING, nodeId);
			ArrayAccessor aa = na.getFieldAccessor("children");
			int size = aa.size();
			Variant[] result = new Variant[ size ];
			aa.getAll(ID_BINDING, result);
			return result;
		} catch (AccessorConstructionException e) {
			throw new AccessorException( e );
		}
	}
	
}

