/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.type;

import java.util.Set;

import org.simantics.databoard.util.IdentityPair;
import org.simantics.databoard.util.Limit;
import org.simantics.databoard.util.ObjectUtils;
import org.simantics.databoard.util.Range;

public class DoubleType extends NumberType {
	
	public static final DoubleType INSTANCE = new DoubleType();
	
    public DoubleType() {}
    public DoubleType(String unit) {
    	this.unit = unit;
    }
    public DoubleType(String unit, Range range) {
    	this.unit = unit;
    	setRange(range);
    }
    public DoubleType(String unit, String range) {
    	this.unit = unit;
    	setRange(range);
    }
    
    public double minValue() {
    	if (range==null) return -Double.MAX_VALUE;
    	Limit l = range.getLower();
    	double value = l.getValue().doubleValue();
    	
    	// Won't work
    	if (l.isExclusive()) value+= Double.MIN_VALUE;
    	
    	return value;
    }
    
    public double maxValue() {
    	if (range==null) return Double.MAX_VALUE;
    	Limit l = range.getUpper();
    	double value = l.getValue().doubleValue();
    	
    	// Won't work
    	if (l.isExclusive()) value-= Double.MIN_VALUE;
    	
    	return value;
    }
    
    @Override
    protected boolean deepEquals(Object obj,
    		Set<IdentityPair<DataType, DataType>> compareHistory) {
		if (this==obj) return true;
		if (obj instanceof DoubleType == false) return false;
		DoubleType other = (DoubleType) obj;
		return 
			ObjectUtils.objectEquals(other.range, range) &&
			ObjectUtils.objectEquals(other.unit, unit);
	}
	
	@Override
	public int hashCode() {
		return 0x4367654 + 13*ObjectUtils.hashCode(range) + 7*ObjectUtils.hashCode(unit);
	}    

	@Override
	public void accept(Visitor1 v, Object obj) {
	    v.visit(this, obj);        
	}

	@Override
	public <T> T accept(Visitor<T> v) {
	    return v.visit(this);
	}
	
}
