/*******************************************************************************
 *  Copyright (c) 2010 Association for Decentralized Information Management in
 *  Industry THTH ry.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 *
 *  Contributors:
 *      VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.databoard.util.binary;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

/**
 * Writeable context
 * 
 * Common interface for ByteBuffer, byte[], OutputStream, RandomAccessFile.
 * 
 * @see ByteBufferWriteable
 * @see FileWriteable
 * @see OutputStreamWriteable 
 * @author Toni Kalajainen (toni.kalajainen@vtt.fi)
 */
public interface BinaryWriteable {

	/**
	 * Write a byte
	 * 
	 * @param b
	 * @throws IOException
	 */
    void put(byte b)
    throws IOException, IndexOutOfBoundsException;
    
    /**
     * Put n bytes from the remaining of the byte array. 
     * This operation moves the pointer in byte buffer.
     * 
     * @param src
     * @throws IOException
     */
    void put(ByteBuffer src)
    throws IOException, IndexOutOfBoundsException;
    
    /**
     * Put n bytes from the remaining of the byte buffer. 
     * This operation moves the pointer in byte buffer.
     * 
     * @param src
     * @param length
     * @throws IOException
     */
    void put(ByteBuffer src, int length)
    throws IOException, IndexOutOfBoundsException;
    
    void put(byte[] src, int offset, int length)
    throws IOException, IndexOutOfBoundsException;
    
    void put(byte[] src)
    throws IOException, IndexOutOfBoundsException;
    
    void putShort(short value)
    throws IOException, IndexOutOfBoundsException;
    
    void putInt(int value)
    throws IOException, IndexOutOfBoundsException;
    
    void putLong(long value)
    throws IOException, IndexOutOfBoundsException;
    
    void putFloat(float value)
    throws IOException, IndexOutOfBoundsException;
    
    void putDouble(double value)
    throws IOException, IndexOutOfBoundsException;
    	
	ByteOrder order();

	void order(ByteOrder order);

	/**
	 * Flush write buffer 
	 * 
	 * @throws IOException
	 */
	void flush()
	throws IOException;
	
}

